@@ -14,7 +14,7 @@ use Data::Dumper;
 
 our ( @EXPORT_OK, %EXPORT_TAGS );
 
-our $VERSION = '0.11';
+our $VERSION = '0.2';
 
 =pod
 
@@ -24,7 +24,7 @@ Device::Velleman::K8055::Fuse - Communication with the Velleman K8055 USB experi
 
 =head1 VERSION
 
-Version 0.11
+Version 0.2
 
 =head1 ABSTRACT
 
@@ -51,7 +51,9 @@ Using the module, it is possible to set two 5v analog output ports, read from tw
     # clear the analog output
     $dev->ClearAllAnalog();
 
-In order to work with this module, the k8055fs utility must be installed. This utility relies on Fuse, which must also be installed.
+=head1 USAGE
+
+=head2 new(pathToDevice=>'/path/to/device',debug=>1)
 
 =cut
 
@@ -98,7 +100,7 @@ my %default_attrs = (
 
 );
 
-=head2 new()
+=head2 $dev new(%attributes)
 
 The constructor. Buils the object.
 
@@ -130,11 +132,6 @@ pathToDevice : the name of the path where the k8055fs commands are mounted.
 
 Returns the object on success
 
-testHarness = 0/1 : Use a test harness rather than the card itself. This allows debugging of the applicaiton logic without relying on the hardware itself being present.
-
-When the test harness is activated, option test => 1 is automatically passed to the InitialiseDevice method.
-
-Furthermore, any Set functionality returns the set value or array of the value, as relevant. Any get function returns -1.
 
 =cut
 
@@ -157,7 +154,6 @@ sub new ($;@) {
 	$dev->InitDevice( $dev->{initDevice} );
 
     } else {
-	unless ($self->{testHarness}) {
 	#check for the existance of the directory
 	warn("Mount point [$dev->{pathToDevice}] does not exist") 
 		unless -d $dev->{pathToDevice};
@@ -166,27 +162,18 @@ sub new ($;@) {
     	warn("Mount point [$dev->{pathToDevice}] is not writable by user")
       		unless -w $dev->{pathToDevice};
 	}
-	}
 
 
     return $dev;
 }
 
-=head2 ReadAnalogChannel();
+=head2 $string ReadAnalogChannel($channel);
 
- my $val1 = $dev->ReadAnalogChannel(1);
- my $val2 = $dev->ReadAnalogChannel(2);
-
- my $channel = 1;
- my $val3 = $dev->ReadAnalogChannel($channel);
-
-
-
-Reads the value from the analog channel indicated by (1 or 2).
+This reads the value from the analog channel indicated by $channel (1 or 2).
 The input voltage of the selected 8-bit Analogue to Digital converter channel is converted to a value
 which lies between 0 and 255.
 
-Returns the numeric  value.
+Returns the string containing the value.
 
 =cut
 
@@ -197,15 +184,9 @@ sub ReadAnalogChannel ($$) {
     return $res;
 }
 
-=head2 ReadAllAnalog();
+=head2 ($one,$two)  ReadAllAnalog();
 
- my ($val1,$val2) = $dev->ReadAllAnalog();
-
-ReadAllAnalog reads the values from the two analog ports into $data1 and $data2.
-
-Inputs: None
-
-Outputs: array of two numeric values
+This reads the values from the two analog ports into $data1 and $data2.
 
 =cut
 
@@ -219,21 +200,15 @@ sub ReadAllAnalog ($) {
     return ( $one, $two );
 }
 
-=head2 OutputAnalogChannel();
-
- my $val = $dev->OutputAnalogChannel(1,0);
- my $val = $dev->OutputAnalogChannel(2,255);
+=head2 $result OutputAnalogChannel($channel, $value);
 
 This outputs $value to the analog channel indicated by $channel.
-
 The indicated 8-bit Digital to Analogue Converter channel is altered according to the new value.
 This means that the value corresponds to a specific voltage. The value 0 corresponds to a
 minimum output voltage (0 Volt) and the value 255 corresponds to a maximum output voltage (+5V).
 A value of $value lying in between these extremes can be translated by the following formula :
 $value / 255 * 5V.
 
-See also SetAnalogChannel() and SetAllAnalog()
-
 =cut
 
 sub OutputAnalogChannel($$) {
@@ -243,64 +218,42 @@ sub OutputAnalogChannel($$) {
     $self->set( "analog_out" . $cid, $val );
 }
 
-=head2 OutputAllAnalog();
-
- my ($val1,$val2) = $dev->OutputAllAnalog(0,255);
-
- my $val = $dev->OutputAllAnalog(255);
+=head2 ($value, $value2) OutputAllAnalog();
 
 This outputs $value1 to the first analog channel, and $value2 to the
-second analog channel. If only one argument is passed, then both channels are given the same value.
-
-See also: SetAllAnalog()
+second analog channel. See OutputAnalogChannel for more information.
 
 =cut
 
 sub OutputAllAnalog($@) {
     my $self = shift;
     my $val1 = shift;
-    my $val2;
- 
-    if (scalar @_) {$val2= shift;} else {$val2 = $val1}
-
+    my $val2 = shift;
     my $cid;
-    my @out;
     $cid = 1;
-    push @out, $self->set( "analog_out" . $cid, $val1 );
+    $self->set( "analog_out" . $cid, $val1 );
     $cid = 2;
-    push @out, $self->set( "analog_out" . $cid, $val2 );
-
-    if ( $val1 != $val2 ) {return @out} else {return $out[0]};
+    $self->set( "analog_out" . $cid, $val2 );
 }
 
-=head2 ClearAnalogChannel();
+=head2 $value ClearAnalogChannel($channel);
 
-This clears the analog channel indicated by $channel. The selected DA-channel is set to minimum output voltage (0 Volt).
-
-Input: channel number
-
-Output: value between 0 (min) and 255 (max)
-
- my $dev->ClearAnalogChannel(1);
- my $dev->ClearAnalogChannel(2);
-
-See also OutputAnalogChannel(), ClearAllAnalog()
+This clears the analog channel indicated by $channel. The selected DA-channel is set to
+minimum output voltage (0 Volt).
 
 =cut
 
 sub ClearAnalogChannel($$) {
     my $self = shift;
     my $cid  = shift;
-    $self->OutputAnalogChannel( $cid, 0 );
+    $self->SetAnalogChannel( $cid, 0 );
 }
 
-=head2 ClearAllAnalog();
+=head2 ($value1,$value1) ClearAllAnalog();
 
 The two DA-channels are set to the minimum output voltage (0 volt).
 
-Returns 0 on success. returns undef if either of the analog channels failed.
-
- my $dev->ClearAllAnalog();
+Returns 255 on success. returns undef if either of the analog channels failed.
 
 =cut
 
@@ -308,27 +261,25 @@ sub ClearAllAnalog($) {
     my $self = shift;
     my $cid;
     $cid = 1;
-    my $one = $self->OutputAnalogChannel( $cid, 0 );
+    my $one = $self->SetAnalogChannel( $cid, 0 );
     $cid = 2;
-    my $two = $self->OutputAnalogChannel( $cid, 0 );
+    my $two = $self->SetAnalogChannel( $cid, 0 );
     return undef if $one == undef || $two == undef;
     return 0;
 }
 
-=head2 SetAnalogChannel();
+=head2 $value SetAnalogChannel($channel);
 
-Sets the selected 8-bit Digital output, which in turns sets the DAC voltage.
-Returns the set value (255) corresponding to this voltage.
-
- my $channel = 1;
- my $val = $dev->SetAllAnalog($channel);
+The selected 8-bit Digital to Analogue Converter channel is set to maximum output voltage.
+Returns the set value.
 
 =cut
 
 sub SetAnalogChannel($$$) {
     my $self = shift;
     my $cid  = shift;
-    $self->set( "analog_out" . $cid, 255 );
+    my $val  = shift;
+    $self->set( "analog_out" . $cid, $val );
 }
 
 =head2 SetAllAnalog();
@@ -336,31 +287,22 @@ sub SetAnalogChannel($$$) {
 The two DA-channels are set to the maximum output voltage.
 Returns 255 on success. returns undef if either of the analog channels failed.
 
- my $val = $dev->SetAllAnalog();
-
 =cut
 
 sub SetAllAnalog ($) {
     my $self = shift;
-    my $one  = $self->OutputAnalogChannel( 1, 255 );
-    my $two  = $self->OutputAnalogChannel( 2, 255 );
+    my $one  = $self->SetAnalogChannel( 1, 255 );
+    my $two  = $self->SetAnalogChannel( 2, 255 );
     return undef unless ( $one && $two );
     return 255;
 }
 
-=head2 WriteAllDigital();
+=head2 WriteAllDigital($value);
 
 The channels of the digital output port are updated with the status of the corresponding
 bits in the $value parameter. A high (1) level means that the microcontroller IC1 output
 is set, and a low (0) level means that the output is cleared.
-
-$value is a decimal value between 0 and 255 that is sent to the output port (8 channels).
-
- # set all 8 digital outputs to 1.
- my $val = 1;
- $res = $dev->WriteAllDigital($val);
-
-Returns the value on success, returns undef on error.
+$value is a value between 0 and 255 that is sent to the output port (8 channels).
 
 =cut
 
@@ -370,18 +312,13 @@ sub WriteAllDigital ($$) {
     $self->set( "digital_out", $val );
 }
 
-=head2 ClearDigitalChannel();
+=head2 ClearDigitalChannel($channel);
 
 This clears the digital output channel $channel, which can have a value between 1 and 8
 that corresponds to the output channel that is to be cleared.
 
 This is the opposite of SetDigitalChannel.
 
- # set digital channel 3  to 0
- $res = $dev->ClearDigitalChannel(3);
-
-Returns 0 on success, undef on error.
-
 =cut
 
 sub ClearDigitalChannel ($$) {
@@ -394,11 +331,6 @@ sub ClearDigitalChannel ($$) {
 
 This clears (sets to 0) all digital output channels.
 
- # set all digital channels  to 0
- $res = $dev->ClearAllDigital();
-
-Returns 0 on success, undef on error.
-
 =cut
 
 sub ClearAllDigital ($) {
@@ -409,7 +341,7 @@ sub ClearAllDigital ($) {
     return 0;
 }
 
-=head2 SetDigitalChannel();
+=head2 SetDigitalChannel($channel);
 
 
 This sets the digital output channel $channel, which can have a value between 1 and 8
@@ -417,9 +349,6 @@ that corresponds to the output channel that is to be cleared.
 
 This is the opposite of ClearDigitalChannel.
 
- # set digital channel 3  to 1
- $res = $dev->SetDigitalChannel(3);
-
 =cut
 
 sub SetDigitalChannel ($$) {
@@ -428,18 +357,9 @@ sub SetDigitalChannel ($$) {
     return $self->AssignDigitalChannel( $cid, 1 );
 }
 
-=head2 AssignDigitalChannel();
+=head2 AssignDigitalChannel($channel,$value,[$digitalFlag]);
 
-This assigns a value todigital channel $channel to the assigned value.
-
- # set digital channel $channel to binary value $value
- $res = $dev->AssignDigitalChannel($channel,$value);
- 
- # set digital channel 3  to 1
- $res = $dev->AssignDigitalChannel(3,1);
-
- # set digital channel 5  to 0
- $res = $dev->AssignDigitalChannel(5,0);
+This sets digital channel $channel to the assigned value.
 
 =cut 
 
@@ -496,8 +416,7 @@ sub AssignDigitalChannel ($$$) {
 
 This sets all digital output channels to 1 (true).
 
- # set digital channels to 1
- $res = $dev->SetAllDigital();
+$obj->SetAllDigital();
 
 sets all digital output channels to 1, giving '1111111'.
 
@@ -516,19 +435,13 @@ sub SetAllDigital ($) {
     return 1;
 }
 
-=head2 ReadAllDigital()
+=head2 $value  ReadAllDigital('bin') or ReadAllDigital();
 
 This reads all 5 digital ports at once. The 5 least significant bits correspond to the
 status of the input channels. A high (1) means that the channel is set, a low (0) means that the channel is cleared.
 Returns the decimal value of the the 8-channel interface card (0-255) unless flag 'bin' is set.
 
-If input contains one string with content  'bin' , then returns an array of binary characters (0/1).
-
- # Get the value of all digital input channels as an array of binary values in big-endian order 
- $res = $dev->ReadAllDigital();
-
- # Get the value of all digital input channels as a decimal value
- $res = $dev->ReadAllDigital('dec');
+If flag 'bin' is set, then returns an array of binary characters (0/1).
 
 Returns undef on error.
 
@@ -557,7 +470,7 @@ sub ReadAllDigital ($;$) {
 
 }
 
-=head2 ReadDigitalChannel();
+=head2 $value = ReadDigitalChannel($channel);
 
 The status of the selected input $channel is read.
 
@@ -568,42 +481,6 @@ The return value will be true (1) if the channel has been set, false (0) otherwi
 
 returns undef on error.
  
- # Get the value of a digital input channel
- $res = $dev->ReadDigitalChannel(1);
-	    
-Note: on the K8055, the addresses of digital inputs 1-5 are not the equivalent binary values.
-
-Refer to the hash $dev->{I} giving the mappings between the card digital input number I and equivalent decimal and binary value, and bit number. $dev->{I} is defined in the constructor.
-
- $dev->{I} = {
-
-        #decimal value vs I-number
-        i2d => {
-            1 => 16,
-            2 => 32,
-            3 => 1,
-            4 => 64,
-            5 => 128,
-        },
-
-        #binary value vs I-number
-        i2b => {
-            1 => '10000',
-            2 => '100000',
-            3 => '1',
-            4 => '1000000',
-            5 => '10000000'
-        },
-
-        #bit number (0-7) value vs I-number
-        i2i => {
-            1 => '4',
-            2 => '5',
-            3 => '0',
-            4 => '6',
-            5 => '7',
-        }
-
 =cut
 
 sub ReadDigitalChannel ($$) {
@@ -621,7 +498,7 @@ sub ReadDigitalChannel ($$) {
     return $val;
 }
 
-=head2 ReadCounter();
+=head2 $value = ReadCounter($counternumber);
 
 The function returns the status of the selected 16 bit pulse counter.
 The counter number 1 counts the pulses fed to the input I1 and the counter number 2 counts the
@@ -631,9 +508,6 @@ returns an 16 bit count on success.
 
 returns undef on error.
 
- my $count = $dev->ReadCounter(1);
- my $count = $dev->ReadCounter(2);
-
 =cut
 
 sub ReadCounter ($$) {
@@ -642,15 +516,12 @@ sub ReadCounter ($$) {
     $self->get( "counter" . $cid );
 }
 
-=head2 ResetCounter();
+=head2 $value = ResetCounter($counternumber);
 
 This resets the selected pulse counter.
 
 returns undef on error.
 
- $my val = $dev->ResetCounter(1);
- $my val = $dev->ResetCounter(2);
-
 =cut
 
 sub ResetCounter ($$) {
@@ -659,7 +530,7 @@ sub ResetCounter ($$) {
     $self->set( "counter" . $cid, 0 );
 }
 
-=head2 SetCounterDebounceTime();
+=head2 $time SetCounterDebounceTime($counternumber, $debouncetime);
 
 The counter inputs are debounced in the software to prevent false triggering when mechanical
 switches or relay inputs are used. The debounce time is equal for both falling and rising edges. The
@@ -674,12 +545,6 @@ returns the set time in milliseconds on success.
 
 returns undef on error.
 
- #set the debounce time for counter 2 to 500ms 
- $my time = $dev->SetCounterDebounceTime(2,500);
-
- #set the debounce time for counter 1 to 2 seconds
- $my time = $dev->SetCounterDebounceTime(1,2000);
-
 =cut
 
 sub SetCounterDebounceTime($$$) {
@@ -695,20 +560,43 @@ sub SetCounterDebounceTime($$$) {
     $self->set( "debounce" . $cid, $time );
 }
 
-=head2 get()
+=head1 AUTHOR
+
+Ronan Oger, C<< <ronan@cpan.org> >>
+
+=head1 ACKNOWLEDGEMENTS
+
+Special thanks to Jouke Visser, author of Device::Velleman::K8055 for writing the original win32-based module. I extensively copied his documentation and derived the method names from the names used by Jouke.
+
+=head1 BUGS
+
+Likely to be many, please use http://rt.cpan.org/ for reporting bugs.
+
+=head1 SEE ALSO
+
+For more information on this board, visit http://www.velleman.be.
+
+For more information on the K0855fs fuse implementation of K0855, visit  https://launchpad.net/k8055fs
+
+For more information on the Fuse driver, visit the FUSE project on sourceforge: http://fuse.sourceforge.net
+
+For Win32 applications, see Jouke Visser's Device::Velleman::K8055 implementation.
+
+=head1 COPYRIGHT & LICENSE
+
+Copyright 2008 Ronan Oger, All Rights Reserved.
+
+This program is free software; you can redistribute it and/or modify it
+under the same terms as Perl itself.
+
+=cut
+
+=head2 get($file,$value)
 
 uses IO::File to retrieve data from the FUSE files. Refer to the k8055fs readme for details.
 
-  my $res = $dev->get('digital_in',255);
-  my $res = $dev->get('analog_in1',255);
-  my $res = $dev->get('analog_in2',255);
-  my $res = $dev->get('counter1',255);
-  my $res = $dev->get('counter2',255);
-
 This is a low-level call that is not particualrly intended for direct access from the API.
 
-The path to the command is defined by hash key pathToDevice in the constructor.
-
 Returns $value on success and undef on error.
 
 =cut
@@ -718,10 +606,9 @@ sub get ($$) {
     my $mfile = shift;
     my $fh    = new IO::File;
     my $res   = undef;
-
     my $file  = $self->{pathToDevice} . "/$mfile";
 
-    if ( !$self->{testHarness} && $fh->open("< $file") ) {
+    if ( $fh->open("< $file") ) {
         my @io = <$fh>;
         $fh->close;
         if ( scalar(@io) != 1 ) {
@@ -732,11 +619,7 @@ sub get ($$) {
         chomp $res;
         $self->{io}->{$mfile} = $res;
         return $res;
-    } elsif ($self->{testHarness}) {
-	$res = -1;
-        $self->{io}->{$mfile} = $res;
-        return $res;
-    }	
+    }
 
     die "02 get: $file: failed. $!\n";
     $self->{io}->{$mfile} = undef;
@@ -747,19 +630,7 @@ sub get ($$) {
 
 uses IO::File to send io to the FUSE files. Refer to the k8055fs readme for details.
 
-  my $res = $dev->set('digital_out',255);
-  my $res = $dev->set('analog_out1',255);
-  my $res = $dev->set('debounce1',255);
-  my $res = $dev->set('debounce2',255);
-
-
 This is a low-level call that is not particualrly intended for direct access from the API.
-Using the set function could desynchronize the internal representation for the binary array
-held in array 
-  
-  $dev->{binary_out}
-
-The path to the command is defined by hash key pathToDevice in the constructor.
 
 Returns $value on success and undef on error.
 
@@ -769,45 +640,27 @@ sub set ($$$) {
     my $self  = shift;
     my $mfile = shift;
     my $val   = shift;
-
-    $val = "-1" unless defined $val;
+    $val = "0" unless defined $val;
 
     my $fh = new IO::File;
 
     my $file = $self->{pathToDevice} . "/$mfile";
 
-    if ( !$self->{testHarness} && $fh->open("> $file") ) {
+    if ( $fh->open("> $file") ) {
         print "set: $file: $val\n" if $self->{'debug'};
         print $fh $val;
         $fh->close;
         chomp $val;
         $self->{io}->{$mfile} = $val;
         return $self->{io}->{$mfile};
-    } elsif ( $self->{testHarnes} ) {
-        $self->{io}->{$mfile} = $val;
-	return $val;
     }
 
     die "01 set: $file: failed. Unable to open file handle: $!\n";
+    $self->{io}->{$mfile} = undef;
+    return $self->{io}->{$mfile};
 }
 
 #from Perl Cookbook (Oreilly)
-
-=head2 dec2bin($dec) 
-
-convert a decimal to a string representing a bin
-
-The binary string is represented as a big-endian. In big-endian encoding, digits increase as the string progresses to the left:
-
-   0 (dec) =        0 (bin).
-   1 (dec) =        1 (bin).
-   2 (dec) =       10 (bin).
-   3 (dec) =       11 (bin).
-   4 (dec) =      100 (bin).
- 255 (dec) = 11111111 (bin).
-
-=cut
-
 sub dec2bin ($$) {
     my $self = shift;
     my $dec  = shift || 0;
@@ -816,21 +669,13 @@ sub dec2bin ($$) {
     return $str;
 }
 
-=head2 bin2dec($bin) 
-
-convert a string representing a binary number  to a  decimal number.
-
-Refer to dec2bin for information on the binary format in use.
-
-=cut
-
 sub bin2dec ($$) {
     my $self = shift;
     my $bin  = shift;
     return unpack( "N", pack( "B32", substr( "0" x 32 . $bin, -32 ) ) );
 }
 
-=head2 InitDevice (\%args)
+=head2 $status InitDevice (\%args)
 
 Initialises the k8055 USB device by mounting the k8055 file system.
 
@@ -845,10 +690,10 @@ Input arguments
 
 pathToDevice: Desired mount point of the k8055fs application.  This directory needs to be accessible by the user.
 
-fuseOptions: additional options to pass to FUSE.
+-fuseOptions: additional options to pass to FUSE.
 
 
-test: do not run the k8055fs initialiaation. Print the command to STDOUT and return success. This is for debugging support.
+-test: do not run the k8055fs initialiaation. Print the command to STDOUT and return success. This is for debugging support.
 
 See also new().
 
@@ -871,10 +716,6 @@ sub InitDevice ($$) {
     $self->{pathToDevice} = $p->{pathToDevice};
 
 
-#Allow us to use the testHarness functionality without actually having the card plugged in.
-#This needs to automatically also invoke the test option.
-
-unless ($self->{testHarness}) {
 	#check for the existance of the directory
     warn("Mount point [$self->{pathToDevice}] does not exist") 
 	unless -d $self->{pathToDevice};
@@ -883,9 +724,6 @@ unless ($self->{testHarness}) {
     warn("Mount point [$self->{pathToDevice}] is not writable by user")
       unless -w $self->{pathToDevice};
 
-} else {
-	$p->{test} = 1;
-}
 
     #see k8055fs README
     my $commands =
@@ -928,35 +766,4 @@ unless ($self->{testHarness}) {
     return;
 }
 
-=head1 AUTHOR
-
-Ronan Oger, C<< <ronan@cpan.org> >>
-
-=head1 ACKNOWLEDGEMENTS
-
-Special thanks to Jouke Visser, author of Device::Velleman::K8055 for writing the original win32-based module. I extensively copied his documentation and derived the method names from the names used by Jouke.
-
-=head1 BUGS
-
-Likely to be many, please use http://rt.cpan.org/ for reporting bugs. The counter functionality is poorly tested and I suspect it has bugs.
-
-=head1 SEE ALSO
-
-For more information on this board, visit http://www.velleman.be.
-
-For more information on the K0855fs fuse implementation of K0855, visit  https://launchpad.net/k8055fs
-
-For more information on the Fuse driver, visit the FUSE project on sourceforge: http://fuse.sourceforge.net
-
-For Win32 applications, see Jouke Visser's Device::Velleman::K8055 implementation.
-
-=head1 COPYRIGHT & LICENSE
-
-Copyright 2008 Ronan Oger, All Rights Reserved.
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
-
-=cut
-
-1
+1;
